/* cpop3.cc
**
**	POP3 class implementation
*/

#include <stdio.h>
#include <unistd.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include "cpop3.h"

CPop3 :: CPop3 (void)
{
}

int CPop3 :: Authenticate (char * user, char * password)
{
char CmdBuf [30];
int result = 0;

    sprintf (CmdBuf, "%s %s\n", POP_USER_COMMAND, user);
    Write (CmdBuf, strlen (CmdBuf));
    if ((result = AwaitResult()) != 0)
	return (result);
    sprintf (CmdBuf, "%s %s\n", POP_PASS_COMMAND, password);
    Write (CmdBuf, strlen (CmdBuf));
    result = AwaitResult();
    return (result);
}

int CPop3 :: Initialize (char * ServerName)
{
int RetryCount = 0;

    while (Open (ServerName, POP_PORT) < 0 && RetryCount < MAX_RETRIES) {
	++ RetryCount;
	sleep (2);
    }

    if (RetryCount == 10) {
    	cerr << "Aborting after " << MAX_RETRIES << " attempts...\n";
	return (-1);
    }
    else	// Read the banner message
	return (AwaitResult());
}

int CPop3 :: AwaitResult (void)
{
char buffer [256];
int status = -1;

    ReadLine (buffer, 256 * sizeof(char));
    if (strstr (buffer, POP_OK_STRING) != NULL)
	status = 0;
    else if (strstr (buffer, POP_ERROR_STRING) != NULL) {
	cerr << "POP error: ";
	cerr << (char*) buffer + strlen(POP_ERROR_STRING) + 1;
	cerr << "\n";
	status = -1;
    }
    return (status);
}

void CPop3 :: GetBoxStats (int * nMessages, int * TotalSize)
{
char CmdBuf [256];
char * TokPtr, * NMPtr, * BSPtr;

    sprintf (CmdBuf, "%s\n", POP_STAT_COMMAND);
    Write (CmdBuf, strlen(CmdBuf));
    ReadLine (CmdBuf, 256 * sizeof(char));
    if (strstr (CmdBuf, POP_OK_STRING) != NULL) {
	// The first one is the OK string
	TokPtr = strtok (CmdBuf, " ");
	// ... followed by the number of messages
	NMPtr = strtok (NULL, " ");
	// ... followed by the total box size
	BSPtr = strtok (NULL, " ");

	// Need to have only numbers in there, so insert zero chars
	*(BSPtr - 1) = '\x0';

	// Now a simple conversion will do.
	*nMessages = atoi (NMPtr);
	*TotalSize = atoi (BSPtr);
    }
    else if (strstr (CmdBuf, POP_ERROR_STRING) != NULL) {
	cerr << "POP error: ";
	cerr << (char*) CmdBuf + strlen(POP_ERROR_STRING) + 1;
	cerr << "\n";
	*nMessages = 0;
	*TotalSize = 0;
    }
}

void CPop3 :: GetMessageStats (int Message, int * Size)
{
char CmdBuf [256];
char * TokPtr;

    sprintf (CmdBuf, "%s %d\n", POP_LIST_COMMAND, Message + 1);
    Write (CmdBuf, strlen(CmdBuf));
    ReadLine (CmdBuf, 256 * sizeof(char));
    if (strstr (CmdBuf, POP_OK_STRING) != NULL) {
	// The first one is the OK string
	TokPtr = strtok (CmdBuf, " ");
	// ... followed by the message number
	TokPtr = strtok (NULL, " ");
	// ... followed by the message size
	TokPtr = strtok (NULL, " ");

	// Now a simple conversion will do.
	*Size = atoi (TokPtr);
    }
    else if (strstr (CmdBuf, POP_ERROR_STRING) != NULL) {
	cerr << "POP error: ";
	cerr << (char*) CmdBuf + strlen(POP_ERROR_STRING) + 1;
	cerr << "\n";
	*Size = 0;
    }
}

void CPop3 :: GetMessage (int Message, char * buffer, int bufsize)
{
char RetrCommand [15];
char MessageLineBuf [256];
int CurLineSize, CurMsgSize = 0;

    sprintf (RetrCommand, "%s %d\n", POP_RETRIEVE_COMMAND, Message + 1);
    Write (RetrCommand, strlen(RetrCommand));
    if (AwaitResult() < 0)
	return;

    memset (MessageLineBuf, 0, 256);
    do {
	CurLineSize = ReadLine (MessageLineBuf, 256);
	if (CurMsgSize + CurLineSize >= bufsize)
	    CurLineSize = bufsize - CurMsgSize;
	memcpy (&buffer [CurMsgSize], MessageLineBuf, CurLineSize);
	CurMsgSize += CurLineSize;
    }
    while (strcmp (MessageLineBuf,POP_END_STRING) != 0 && CurMsgSize < bufsize);
}

void CPop3 :: DeleteMessage (int Message)
{
char CmdBuf [80];
    sprintf (CmdBuf, "%s %d\n", POP_DELETE_COMMAND, Message + 1);
    Write (CmdBuf, strlen (CmdBuf));
    AwaitResult();
}

void CPop3 :: Shutdown (void)
{
char CmdBuf [30];
    sprintf (CmdBuf, "%s\n", POP_QUIT_COMMAND);
    Write (CmdBuf, strlen (CmdBuf));
    AwaitResult();
    Close();
}

CPop3 :: ~CPop3 (void)
{
}

