/* main.cc
**
**	Main module for a POP3 message retrieval client.
*/

#define SENDMAIL_COMMAND	"/usr/lib/sendmail"

#include <iostream.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "cpop3.h"

/*-----------------------------------------------------------------*/
  int 	ReadConfigFile (void);
  int 	DeliverMessage (char * MsgBuf, int MsgSize);
/*-----------------------------------------------------------------*/

struct {
    char mailhost [40];
    char user [17];
    char password [17];
} PopConfig;

int main (int argc, char ** argv)
{
CPop3 PopClient;
int nMessages, TotalSize, MessageSize, i;
char * MessageBuffer;
int DeliverySuccessful = 1;

    if (argc > 1) {
	cout << argv[0] << " - a POP3 mail reader version 0.1 ALPHA\n";
	cout << "Written by Mike Sharov in cold January 1996\n\n";
	return (0);
    }

    if (ReadConfigFile() != 0)
	return (1);

    cout << "Connecting to " << PopConfig.mailhost << ", please wait.\n";

    if (PopClient.Initialize (PopConfig.mailhost) != 0) {
	cerr << "Could not connect to mail host " << PopConfig.mailhost << "\n";
	return (1);
    }
    if (PopClient.Authenticate (PopConfig.user, PopConfig.password) != 0) {
	cerr << "Username " << PopConfig.user << " is invalid\n";
	return (1);
    }

    PopClient.GetBoxStats (&nMessages, &TotalSize);

    if (nMessages == 0) {
	cout << "No mail.\n";
	return (0);
    }

    cout << "Reading " << nMessages << " messages ";
    cout.flush();
    for (i = 0; i < nMessages; ++ i) {
	PopClient.GetMessageStats (i, &MessageSize);
	// Take zero terminator in account
	++ MessageSize;
	MessageBuffer = (char*) malloc (MessageSize * sizeof(char));

	PopClient.GetMessage (i, MessageBuffer, MessageSize);

	// Don't delete unless sendmail call was successful
	if (DeliverMessage (MessageBuffer, MessageSize) != 0) {
	    cerr << "Aborting message download. Stand by.\n";
	    cerr.flush();
	    i = nMessages;
	    DeliverySuccessful = 0;
	}
	
	free (MessageBuffer);
	cout << ".";
	cout.flush();
    }

    if (DeliverySuccessful)
	for (i = 0; i < nMessages; ++ i)
	    PopClient.DeleteMessage (i);	// Index changes...
    sleep (1);
    PopClient.Shutdown();

    cout << "\nAll done.\n";

    return (0);
}

int ReadConfigFile (void)
{
FILE * fp;
char filename [80];

    strcpy (filename, getenv ("HOME"));
    strcat (filename, "/.pop");
    if ((fp = fopen (filename, "r")) == NULL) {
	cerr << "Could not open config file $HOME/.pop\n";
	return (-1);
    }

    fscanf (fp, "%s %s %s", PopConfig.mailhost, PopConfig.user, 
    			    PopConfig.password);
    fclose (fp);

    return (0);
}

int DeliverMessage (char * MsgBuf, int MsgSize)
{
FILE * fp;
char SendmailCommand [80];

    // Setup to send the message to the person running this program
    strcpy (SendmailCommand, SENDMAIL_COMMAND);
    strcat (SendmailCommand, " ");
    strcat (SendmailCommand, getlogin());

    // ..., using a direct pipe to sendmail
    if ((fp = popen (SendmailCommand, "w")) == NULL) {
	cerr << "\nCould not start sendmail!\n";
	return (-1);
    }

    fwrite (MsgBuf, MsgSize, 1, fp);
    pclose (fp);

    return (0);
}

